

from skipole import FailPage, ValidateError, GoTo, ServerError, skilift

from skipole.skilift import editpage, editsection


def show_empty_modal_insert(skicall):
    "Fills in empty modal insert"
    call_data = skicall.call_data
    page_data = skicall.page_data
    page_data["widgetinserts", "insertitem", "hide"] = False
    location = call_data['location']
    widget_name = location[0]
    container = location[1]
    # This should be for an empty container
    if location[2]:
        raise FailPage("Invalid location")

    insert_location = widget_name + "-" + str(container)

    # for each of the links, set get_field1 to be the insert_location
    page_data[("widgetinserts","insert_text","get_field1")] = insert_location
    page_data[("widgetinserts","insert_textblock","get_field1")] = insert_location
    page_data[("widgetinserts","insert_symbol","get_field1")] = insert_location
    page_data[("widgetinserts","insert_comment","get_field1")] = insert_location
    page_data[("widgetinserts","insert_element","get_field1")] = insert_location
    page_data[("widgetinserts","insert_widget","get_field1")] = insert_location
    page_data[("widgetinserts","insert_section","get_field1")] = insert_location

    # set the hidden field
    page_data[("widgetinserts","uploadpart","hidden_field1")] = insert_location



def insert_in_widget(skicall):
    """Called by domtable to either insert or append an item in a widget container
       sets page_data to populate the insert or append modal panel"""
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
    elif "section_name" in call_data:
        section_name = call_data["section_name"]
    else:
        raise FailPage(message = "No page or section given")

    if ('editdom', 'domtable', 'contents') not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data['editdom', 'domtable', 'contents']

    # so part is location_string with string of integers
    # where first string is the widget name,
    # and first digit is the container number
    # create location which is a tuple or list consisting of three items:
    # widget name
    # a container integer
    # a tuple or list of location integers
    location_list = part.split('-')
    # first item should be a string, rest integers
    # first item should be a string, rest integers
    if len(location_list) < 3:
        raise FailPage("Item to append to has not been recognised")
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    insert_location = widget_name + '-' + str(container) + '-' + '-'.join(str(i) for i in location_integers)

    # display the modal panel
    page_data[("widgetinserts","insertitem","hide")] = False

    if (part_tuple.part_type == "Part") or (part_tuple.part_type == "Section"):
        # insert
        page_data[("widgetinserts","insertpara","para_text")] = "Choose an item to insert"
        page_data[("widgetinserts","insertupload","para_text")] = "Or insert a new block by uploading a block definition file:"
    else:
        # append
        page_data[("widgetinserts","insertpara","para_text")] = "Choose an item to append"
        page_data[("widgetinserts","insertupload","para_text")] = "Or append a new block by uploading a block definition file:"

    # for each of the links, set get_field1 to be the insert_location
    page_data[("widgetinserts","insert_text","get_field1")] = insert_location
    page_data[("widgetinserts","insert_textblock","get_field1")] = insert_location
    page_data[("widgetinserts","insert_symbol","get_field1")] = insert_location
    page_data[("widgetinserts","insert_comment","get_field1")] = insert_location
    page_data[("widgetinserts","insert_element","get_field1")] = insert_location
    page_data[("widgetinserts","insert_widget","get_field1")] = insert_location
    if pagenumber:
        page_data[("widgetinserts","insert_section","get_field1")] = insert_location

    # set the hidden field
    page_data[("widgetinserts","uploadpart","hidden_field1")] = insert_location


def insert_text(skicall):
    "Inserts text into a page"
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None
    editedprojname = call_data['editedprojname']

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
        page_info = skilift.item_info(editedprojname, pagenumber)
        if page_info is None:
            raise FailPage("Page to edit not identified")
        if (page_info.item_type != "TemplatePage") and (page_info.item_type != "SVG"):
            raise FailPage("Page not identified")
    elif "section_name" in call_data:
        section_name = call_data["section_name"]
    else:
        raise FailPage(message = "No page or section given")

    if ("widgetinserts","insert_text","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")

    part = call_data["widgetinserts","insert_text","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        if len(location_list) < 3:
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    new_text = 'Set text here'
    if pagenumber:
        call_data['pchange'], new_location = skilift.insert_item_in_page(editedprojname, pagenumber, call_data['pchange'], location, new_text)
        page_data[("adminhead","page_head","large_text")] = "Edit Text in Page: %s Widget: %s" % (pagenumber,widget_name)
    else:
        call_data['schange'], new_location = skilift.insert_item_in_section(editedprojname, section_name, call_data['schange'], location, new_text)
        page_data[("adminhead","page_head","large_text")] = "Edit Text in Section : %s Widget %s" % (section_name,widget_name)
    call_data['location'] = new_location
    
    # go to edit text page, set the text in the text area
    page_data[("text_input","input_text")] = new_text


def insert_textblock(skicall):
    "Fills the template page for creating a textblock reference which will be inserted in the edited page"
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None
    editedprojname = call_data['editedprojname']

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
        page_info = skilift.item_info(editedprojname, pagenumber)
        if page_info is None:
            raise FailPage("Page to edit not identified")
        if (page_info.item_type != "TemplatePage") and (page_info.item_type != "SVG"):
            raise FailPage("Page not identified")
    elif "section_name" in call_data:
        section_name = call_data["section_name"]
    else:
        raise FailPage(message = "No page or section given")

    if ("widgetinserts","insert_textblock","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")

    part = call_data["widgetinserts","insert_textblock","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        if len(location_list) < 3:
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")
    call_data['location'] = location

    # and set page data for the template page which inserts an textblock reference
    if pagenumber:
        page_data[("adminhead","page_head","large_text")] = "Insert TextBlock in Page: %s Widget: %s" % (pagenumber, widget_name)
    else:
        page_data[("adminhead","page_head","large_text")] = "Insert TextBlock in Section: %s Widget: %s" % (section_name, widget_name)

    page_data[("linebreaks","radio_values")]=['ON', 'OFF']
    page_data[("linebreaks","radio_text")]=['On', 'Off']
    page_data[("linebreaks","radio_checked")] = 'ON'

    page_data[("setescape","radio_values")]=['ON', 'OFF']
    page_data[("setescape","radio_text")]=['On', 'Off']
    page_data[("setescape","radio_checked")] = 'ON'


def insert_symbol(skicall):
    "Inserts html symbol into a page"
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None
    editedprojname = call_data['editedprojname']

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
        page_info = skilift.item_info(editedprojname, pagenumber)
        if page_info is None:
            raise FailPage("Page to edit not identified")
        if (page_info.item_type != "TemplatePage") and (page_info.item_type != "SVG"):
            raise FailPage("Page not identified")
    elif "section_name" in call_data:
        section_name = call_data["section_name"]
    else:
        raise FailPage(message = "No page or section given")

    if ("widgetinserts","insert_symbol","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")

    part = call_data["widgetinserts","insert_symbol","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        if len(location_list) < 3:
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    # then passed to edit symbol page

    if pagenumber:
        call_data['pchange'], new_location = editpage.create_html_symbol_in_page(editedprojname, pagenumber, call_data['pchange'], location)
        sym = editpage.get_symbol(editedprojname, pagenumber, call_data['pchange'], new_location)
        page_data[("adminhead","page_head","large_text")] = "Edit Symbol in Page: %s Widget: %s" % (pagenumber, widget_name)
    else:
        call_data['schange'], new_location = editsection.create_html_symbol_in_section(editedprojname, section_name, call_data['schange'], location)
        sym = editsection.get_symbol(editedprojname, section_name, call_data['schange'], new_location)
        page_data[("adminhead","page_head","large_text")] = "Edit Symbol in Section: %s Widget: %s" % (section_name, widget_name)

    call_data['location'] = new_location
    page_data["symbol_input","input_text"] = sym


def insert_comment(skicall):
    "Inserts a comment into a page"
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None
    editedprojname = call_data['editedprojname']

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
        page_info = skilift.item_info(editedprojname, pagenumber)
        if page_info is None:
            raise FailPage("Page to edit not identified")
        if (page_info.item_type != "TemplatePage") and (page_info.item_type != "SVG"):
            raise FailPage("Page not identified")
    elif "section_name" in call_data:
        section_name = call_data["section_name"]
    else:
        raise FailPage(message = "No page or section given")

    if ("widgetinserts","insert_comment","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")


    part = call_data["widgetinserts","insert_comment","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        if len(location_list) < 3:
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    # then passed to edit symbol page

    if pagenumber:
        call_data['pchange'], new_location = editpage.create_html_comment_in_page(editedprojname, pagenumber, call_data['pchange'], location)
        com = editpage.get_comment(editedprojname, pagenumber, call_data['pchange'], new_location)
        page_data[("adminhead","page_head","large_text")] = "Edit Comment in Page: %s Widget: %s" % (pagenumber, widget_name)
    else:
        call_data['schange'], new_location = editsection.create_html_comment_in_section(editedprojname, section_name, call_data['schange'], location)
        com = editsection.get_comment(editedprojname, section_name, call_data['schange'], new_location)
        page_data[("adminhead","page_head","large_text")] = "Edit Comment in Section: %s Widget: %s" % (section_name, widget_name)

    call_data['location'] = new_location
    page_data[("comment_input","input_text")] = com


def insert_element(skicall):
    "Fills the template page for creating an html element which will be inserted in the edited page"
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None
    editedprojname = call_data['editedprojname']

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
        page_info = skilift.item_info(editedprojname, pagenumber)
        if page_info is None:
            raise FailPage("Page to edit not identified")
        if (page_info.item_type != "TemplatePage") and (page_info.item_type != "SVG"):
            raise FailPage("Page not identified")
    elif "section_name" in call_data:
        section_name = call_data["section_name"]
    else:
        raise FailPage(message = "No page or section given")

    if ("widgetinserts","insert_element","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")

    part = call_data["widgetinserts","insert_element","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        if len(location_list) < 3:
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")
    call_data['location'] = location

    # and set page data for the template page which inserts an HTML element
    if pagenumber:
        page_data[("adminhead","page_head","large_text")] = "Insert  an HTML element into Page: %s Widget: %s" % (pagenumber, widget_name)
    else:
        page_data[("adminhead","page_head","large_text")] = "Insert  an HTML element into Section: %s Widget: %s" % (section_name, widget_name)


def insert_widget(skicall):
    "Gets page number and location, used for creating a widget which will be inserted in the page"
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None
    editedprojname = call_data['editedprojname']

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
        page_info = skilift.item_info(editedprojname, pagenumber)
        if page_info is None:
            raise FailPage("Page to edit not identified")
        if (page_info.item_type != "TemplatePage") and (page_info.item_type != "SVG"):
            raise FailPage("Page not identified")
    elif "section_name" in call_data:
        section_name = call_data["section_name"]
    else:
        raise FailPage(message = "No page or section given")

    if ("widgetinserts","insert_widget","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")


    part = call_data["widgetinserts","insert_widget","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        if len(location_list) < 3:
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")
    call_data['location'] = location

    # at this point, the call is passed to 54507 which is a responder which lists widget modules
    # and displays them on a template


def insert_section(skicall):
    "Gets page number and location, used for creating a section reference which will be inserted in the page"
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None
    editedprojname = call_data['editedprojname']

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
    else:
        raise FailPage(message = "No page given")

    if ("widgetinserts","insert_section","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")

    page_info = skilift.item_info(editedprojname, pagenumber)
    if page_info is None:
        raise FailPage("Page to edit not identified")
    if (page_info.item_type != "TemplatePage") and (page_info.item_type != "SVG"):
        raise FailPage("Page not identified")

    part = call_data["widgetinserts","insert_section","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        if len(location_list) < 3:
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, None, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")
    call_data['location'] = location

    # Fill in header
    page_data[("adminhead","page_head","large_text")] = "Insert Section place holder"

    # get current sections
    section_list = editsection.list_section_names(editedprojname)
    if not section_list:
        page_data[("nosection", "show")] = True
        page_data[("descript", "show")] = False
        page_data[("placename","show")] = False
        return

    page_data[('sectionname','option_list')] = section_list[:]
    page_data[('sectionname','selectvalue')] = section_list[0]



def insert_upload(skicall):
    "Gets page number and location, used for creating a widget which will be inserted in the page"
    call_data = skicall.call_data
    page_data = skicall.page_data

    pagenumber = None
    section_name = None
    editedprojname = call_data['editedprojname']

    if "page_number" in call_data:
        pagenumber = call_data["page_number"]
        page_info = skilift.item_info(editedprojname, pagenumber)
        if page_info is None:
            raise FailPage("Page to edit not identified")
        if (page_info.item_type != "TemplatePage") and (page_info.item_type != "SVG"):
            raise FailPage("Page not identified")
    elif "section_name" in call_data:
        section_name = call_data["section_name"]
    else:
        raise FailPage(message = "No page or section given")

    if ("widgetinserts","uploadpart","hidden_field1") not in call_data:
        raise FailPage(message = "item to edit missing")

    part = call_data["widgetinserts","uploadpart","hidden_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        widget_name = location_list[0]
        container = int(location_list[1])
        if len(location_list) < 3:
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[2:] )
    except:
        raise FailPage("Item to append to has not been recognised")

    location = (widget_name, container, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, pagenumber, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")
    # get file contents
    file_contents = call_data["widgetinserts","uploadpart", "action"]
    json_string = file_contents.decode(encoding='utf-8')
    try:
        if pagenumber:
            call_data['pchange'] = editpage.create_part_in_page(editedprojname, pagenumber, call_data['pchange'], location, json_string)
        else:
            call_data['schange'] = editsection.create_part_in_section(editedprojname, section_name, call_data['schange'], location, json_string)
    except ServerError as e:
        if e.message:
            raise FailPage(e.message)
        else:
            raise FailPage("An error has occurred in creating the item")
    call_data['widget_name'] = widget_name
    call_data['container'] = container
    call_data['status'] = 'New block created'





import setuptools

from distutils.core import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="skipole",
    version="5.0.4",
    author="Bernard Czenkusz",
    author_email="bernie@skipole.co.uk",
    description="A WSGI Application generator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bernie-skipole.github.io/skipole/",
    packages=['skipole', 'skipole.skilift', 'skipole.ski', 'skipole.ski.responders', 'skipole.ski.validators', 'skipole.ski.widgets',
              'skipole.skiadmin', 'skipole.skiadmin.skiadminpackages', 'skipole.skiadmin.skiadminpackages.editcss', 'skipole.skiadmin.skiadminpackages.editfiles',
              'skipole.skiadmin.skiadminpackages.editfolders', 'skipole.skiadmin.skiadminpackages.editpages', 'skipole.skiadmin.skiadminpackages.editparts',
              'skipole.skiadmin.skiadminpackages.editresponders', 'skipole.skiadmin.skiadminpackages.editsectionplaces', 'skipole.skiadmin.skiadminpackages.editsections',
              'skipole.skiadmin.skiadminpackages.editspecialpages', 'skipole.skiadmin.skiadminpackages.edittext', 'skipole.skiadmin.skiadminpackages.edittextblocks',
              'skipole.skiadmin.skiadminpackages.editvalidators', 'skipole.skiadmin.skiadminpackages.editwidgets'],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application"
    ],
)





from skipole import skilift
from skipole.skilift import editpage, editsection
from skipole import FailPage, ValidateError, GoTo, ServerError



def retrieve_edittextpage(skicall):
    "Fills in the edit text page"

    call_data = skicall.call_data
    page_data = skicall.page_data

    # a skilift.part_tuple is (project, pagenumber, page_part, section_name, name, location, part_type, brief)

    pagenumber = None
    section_name = None

    if 'part_tuple' in call_data:
        part_tuple = call_data['part_tuple']
        project = part_tuple.project
        location = part_tuple.location
        pagenumber = part_tuple.pagenumber
        section_name = part_tuple.section_name
    else:
        project = call_data['editedprojname']
        location = call_data['location']
        if 'page_number' in call_data:
            pagenumber = call_data['page_number']
        else:
            section_name = call_data['section_name']

    if (pagenumber is None) and (section_name is None):
        raise ValidateError("Page/section not identified")

    call_data['location'] = location

    try:

        if pagenumber:
            call_data['page_number'] = pagenumber
            text = editpage.get_text(project, pagenumber, call_data['pchange'], location)
            page_data[("adminhead","page_head","large_text")] = "Edit Text in page : %s" % (pagenumber,)
        elif section_name:
            call_data['section_name'] = section_name
            text = editsection.get_text(project, section_name, call_data['schange'], location)
            page_data[("adminhead","page_head","large_text")] = "Edit Text in section : %s" % (section_name,)
        else:
            raise FailPage("Either a page or section must be specified")

    except ServerError as e:
        raise FailPage(e.message)

    # Set the text in the text area
    page_data[("text_input","input_text")] = text



def edit_text(skicall):
    "Submits new text"

    call_data = skicall.call_data
    page_data = skicall.page_data

    project = call_data['editedprojname']
    location = call_data['location']

    text = call_data['text']

    try:
        if 'page_number' in call_data:
            pagenumber = call_data['page_number']
            call_data['pchange'] = skilift.set_item_in_page(project, pagenumber, call_data['pchange'], location, text)
        elif 'section_name' in call_data:
            section_name = call_data['section_name']
            call_data['schange'] = skilift.set_item_in_section(project, section_name, call_data['schange'], location, text)
        else:
            raise ValidateError("Page/section not identified")
    except ServerError as e:
        raise FailPage(e.message)
    call_data['status'] = "Text changed"



def retrieve_edit_symbol(skicall):
    "Fills in the edit html symbol page"

    call_data = skicall.call_data
    page_data = skicall.page_data

    # a skilift.part_tuple is (project, pagenumber, page_part, section_name, name, location, part_type, brief)

    pagenumber = None
    section_name = None

    if 'part_tuple' in call_data:
        part_tuple = call_data['part_tuple']
        project = part_tuple.project
        location = part_tuple.location
        pagenumber = part_tuple.pagenumber
        section_name = part_tuple.section_name
    else:
        project = call_data['editedprojname']
        location = call_data['location']
        if 'page_number' in call_data:
            pagenumber = call_data['page_number']
        else:
            section_name = call_data['section_name']

    if (pagenumber is None) and (section_name is None):
        raise ValidateError("Page/section not identified")

    call_data['location'] = location

    try:

        if pagenumber:
            call_data['page_number'] = pagenumber
            sym = editpage.get_symbol(project, pagenumber, call_data['pchange'], location)
            page_data[("adminhead","page_head","large_text")] = "Edit Symbol in page : %s" % (pagenumber,)
        elif section_name:
            call_data['section_name'] = section_name
            sym = editsection.get_symbol(project, section_name, call_data['schange'], location)
            page_data[("adminhead","page_head","large_text")] = "Edit Symbol in section : %s" % (section_name,)
        else:
            raise FailPage("Either a page or section must be specified")

    except ServerError as e:
        raise FailPage(e.message)

    # Set the symbol in the text area
    page_data["symbol_input","input_text"] = sym


def set_edit_symbol(skicall):
    "Submits new symbol after editing"

    call_data = skicall.call_data
    page_data = skicall.page_data

    project = call_data['editedprojname']
    location = call_data['location']

    if 'symbol' not in call_data:
        raise FailPage(message = "Invalid symbol")

    symbol = call_data['symbol']

    try:
        if 'page_number' in call_data:
            pagenumber = call_data['page_number']
            call_data['pchange'] = editpage.edit_page_symbol(project, pagenumber, call_data['pchange'], location, symbol)
        elif 'section_name' in call_data:
            section_name = call_data['section_name']
            call_data['schange'] = editsection.edit_section_symbol(project, section_name, call_data['schange'], location, symbol)
        else:
            raise ValidateError("Page/section not identified")
    except ServerError as e:
        raise FailPage(e.message)
    call_data['status'] = "Symbol changed"


def retrieve_edit_comment(skicall):
    "Fills in the edit html comment page"

    call_data = skicall.call_data
    page_data = skicall.page_data

    # a skilift.part_tuple is (project, pagenumber, page_part, section_name, name, location, part_type, brief)

    pagenumber = None
    section_name = None

    if 'part_tuple' in call_data:
        part_tuple = call_data['part_tuple']
        project = part_tuple.project
        location = part_tuple.location
        pagenumber = part_tuple.pagenumber
        section_name = part_tuple.section_name
    else:
        project = call_data['editedprojname']
        location = call_data['location']
        if 'page_number' in call_data:
            pagenumber = call_data['page_number']
        else:
            section_name = call_data['section_name']

    if (pagenumber is None) and (section_name is None):
        raise ValidateError("Page/section not identified")

    call_data['location'] = location

    try:

        if pagenumber:
            call_data['page_number'] = pagenumber
            com = editpage.get_comment(project, pagenumber, call_data['pchange'], location)
            page_data[("adminhead","page_head","large_text")] = "Edit Comment in page : %s" % (pagenumber,)
        elif section_name:
            call_data['section_name'] = section_name
            com = editsection.get_comment(project, section_name, call_data['schange'], location)
            page_data[("adminhead","page_head","large_text")] = "Edit Comment in section : %s" % (section_name,)
        else:
            raise FailPage("Either a page or section must be specified")

    except ServerError as e:
        raise FailPage(e.message)

    # Set the text in the input field
    page_data[("comment_input","input_text")] = com


def set_edit_comment(skicall):
    "Submits new comment after editing"

    call_data = skicall.call_data
    page_data = skicall.page_data

    project = call_data['editedprojname']
    location = call_data['location']

    if 'comment' not in call_data:
        raise FailPage(message = "Invalid symbol")

    comment = call_data['comment']

    try:
        if 'page_number' in call_data:
            pagenumber = call_data['page_number']
            call_data['pchange'] = editpage.edit_page_comment(project, pagenumber, call_data['pchange'], location, comment)
        elif 'section_name' in call_data:
            section_name = call_data['section_name']
            call_data['schange'] = editsection.edit_section_comment(project, section_name, call_data['schange'], location, comment)
        else:
            raise ValidateError("Page/section not identified")
    except ServerError as e:
        raise FailPage(e.message)
    call_data['status'] = "Comment changed"







"""Validators that deal with path, folder and file names

Each Validator is a class, with a _check method

"""


from .. import tag
from .. import skiboot
from . import Validator
from ..excepts import ValidateError


class PathLeadingTrailingSlashes(Validator):
    "Validates a path, and ensures leading or trailing slashes are in the path"

    arg_descriptions = {'leading_slash':True,
                       'trailing_slash':True }

    def __setitem__(self, arg_name, value):
        "Sets an argument value"
        Validator.__setitem__(self, arg_name, value)
        if isinstance(value, str):
            if val.lower() == 'false':
                self._val_args[arg_name] = False
        self._val_args[arg_name] = bool(value)


    def _check(self, widgfield, item, environ, lang, form_data, call_data, caller_page_ident):
        if not item and (self["leading_slash"] or self["trailing_slash"]):
            return "/", True
        if not item:
            return '', False
        # strip leading and trailing slashes
        item = item.strip("/")
        item = item.strip()
        if not item and (self["leading_slash"] or self["trailing_slash"]):
            return "/", True
        if not item:
            return '', False
        if not item[0].isalnum():
            return '', False
        # add leading /
        if self["leading_slash"]:
            item = "/"+item
        # add an end /
        if self["trailing_slash"]:
            item = item+"/"
        return item, True


class IdentExists(Validator):
    "Ensures a given ident exists"

    arg_descriptions = {'is_folder':False,
                       'is_page':True,
                       'can_be_empty':False}

    def __setitem__(self, arg_name, value):
        "Sets an argument value"
        Validator.__setitem__(self, arg_name, value)
        if isinstance(value, str):
            if val.lower() == 'false':
                self._val_args[arg_name] = False
        self._val_args[arg_name] = bool(value)

    def _check(self, widgfield, item, environ, lang, form_data, call_data, caller_page_ident):
        if (not item) and (item != 0) and self["can_be_empty"]:
            # ident item is empty, and this is allowed
            return '', True
        page = skiboot.from_ident(item)
        if page is None:
            return '', False
        if (page.page_type == "Folder") and self["is_folder"]:
            return str(page.ident), True
        if (page.page_type != 'Folder') and self["is_page"]:
            return str(page.ident), True
        return '', False


class NameNotInFolder(Validator):
    """Checks the given name does not exist within the folder,
       which has its ident given in the field 'folder_ident_field'
    """

    arg_descriptions = {'folder_ident_field':''}

    def __setitem__(self, arg_name, value):
        "Sets an argument value"
        Validator.__setitem__(self, arg_name, value)
        if not value:
            self._val_args[arg_name] = ''
        self._val_args[arg_name] = skiboot.make_widgfield(value)


    def _check(self, widgfield, item, environ, lang, form_data, call_data, caller_page_ident):
        # Get the folder
        folder = None
        if self["folder_ident_field"] in form_data:
            folder = skiboot.from_ident(form_data[self["folder_ident_field"]])
        if not folder:
            return '', False
        if folder.page_type != 'Folder':
            return '', False
        # we have the folder, is this item in the folder
        name = item.lower()
        if name in folder:
            return '', False
        return name, True








import os, json, importlib, inspect, collections, uuid


from . import skiboot, excepts, folder_class_definition, page_class_definition, tag



def make_part_for_page(page, json_data):
    "Returns object created from json data, the page argument is used to give any items unique names. The object is not inserted, and page remains unchanged"
    if isinstance(json_data, str):
        part_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        part_dict = json_data
    newpart =  _create_part(part_dict, page.proj_ident)
    # ensure widgets and placeholders in newpart have unique names
    name_list = list(page.widgets.keys()) + list(page.section_places.keys())
    newpart.set_unique_names(name_list)
    return newpart


def make_item_for_page(page, json_string):
    "Returns object created from json string, the page argument is used to give any items unique names. The object is not inserted, and page remains unchanged"
    item_list = json.loads(json_string, object_pairs_hook=collections.OrderedDict)
    newitem = _create_item(item_list, page.proj_ident)
    # ensure widgets and placeholders in newitem have unique names
    if hasattr(newitem, 'set_unique_names'):
        name_list = list(page.widgets.keys()) + list(page.section_places.keys())
        newitem.set_unique_names(name_list)
    return newitem


def make_part_for_section(section, json_data):
    "Returns object created from json data, the section argument is used to give any items unique names. The object is not inserted, and section remains unchanged"
    if isinstance(json_data, str):
        part_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        part_dict = json_data
    newpart =  _create_part(part_dict, section.proj_ident)
    # ensure widgets and placeholders in newpart have unique names
    name_list = list(section.widgets.keys()) + list(section.section_places.keys())
    newpart.set_unique_names(name_list)
    # as this is going into a section, remove any placeholders
    _remove_sectionplaceholders(newpart)
    return newpart


def make_item_for_section(section, json_string):
    "Returns object created from json string, the section argument is used to give any items unique names. The object is not inserted, and section remains unchanged"
    item_list = json.loads(json_string, object_pairs_hook=collections.OrderedDict)
    newitem = _create_item(item_list, section.proj_ident)
    # ensure widgets and placeholders in newitem have unique names
    if hasattr(newitem, 'set_unique_names'):
        name_list = list(section.widgets.keys()) + list(section.section_places.keys())
        newitem.set_unique_names(name_list)
    # as this is going into a section, remove any placeholders
    _remove_sectionplaceholders(newitem)
    return newitem


def _remove_sectionplaceholders(part):
    "Given a part - swap out any tag.SectionPlaceHolder for a text string"
    if isinstance(part, tag.Part) or isinstance(part, tag.Section):
        parts = part.parts
        index_list = []
        for index, item in enumerate(parts):
            if isinstance(item, tag.SectionPlaceHolder):
                index_list.append(index)
            if hasattr(item, 'parts'):
                _remove_sectionplaceholders(item)
        for index in index_list:
            parts[index] = "Section Place Holder Removed"


def create_section(proj_ident, section_name, json_data):
    """Builds the section from the given json string, or ordered dictionary and adds it to project"""
    if isinstance(json_data, str):
        section_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        section_dict = json_data
    section = _create_section(section_dict, proj_ident)
    # as this is going into a section, remove any placeholders
    _remove_sectionplaceholders(section)
    project = skiboot.getproject(proj_ident)
    project.add_section(section_name, section)


def create_templatepage(proj_ident, parent_ident, page_ident, page_name, brief, json_data):
    """Builds the template page from the given json string, or ordered dictionary and adds it to project"""
    if isinstance(json_data, str):
        page_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        page_dict = json_data
    if "TemplatePage" not in page_dict:
        raise excepts.ServerError("Invalid file")
    page_args = page_dict["TemplatePage"]
    page = _create_templatepage(page_name, brief, page_args)
    head = page_dict["head"]
    body = page_dict["body"]
    page.head = _create_item(head, proj_ident)
    page.body = _create_item(body, proj_ident)
    parent = skiboot.from_ident(parent_ident, proj_ident)
    parent.add_page(page, page_ident)


def create_svgpage(proj_ident, parent_ident, page_ident, page_name, brief, json_data):
    """Builds the svg page from the given json string, or ordered dictionary and adds it to project"""
    if isinstance(json_data, str):
        page_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        page_dict = json_data
    if "SVG" not in page_dict:
        raise excepts.ServerError("Invalid file")
    page_args = page_dict["SVG"]
    page = _create_svgpage(page_name, brief, page_args)
    if "svg" in page_dict:
        svg = page_dict["svg"]
        page.svg = _create_item(svg, proj_ident)
    parent = skiboot.from_ident(parent_ident, proj_ident)
    parent.add_page(page, page_ident)


def create_filepage(proj_ident, parent_ident, page_ident, page_name, brief, json_data):
    """Builds the file page from the given json string, or dictionary and adds it to project"""
    if isinstance(json_data, str):
        page_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        page_dict = json_data
    if "FilePage" not in page_dict:
        raise excepts.ServerError("Invalid file")
    page_args = page_dict["FilePage"]
    page = _create_filepage(page_name, brief, page_args)
    parent = skiboot.from_ident(parent_ident, proj_ident)
    parent.add_page(page, page_ident)


def create_csspage(proj_ident, parent_ident, page_ident, page_name, brief, json_data):
    """Builds the css page from the given json string, or ordered dictionary and adds it to project"""
    if isinstance(json_data, str):
        page_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        page_dict = json_data
    if "CSS" not in page_dict:
        raise excepts.ServerError("Invalid file")
    page_args = page_dict["CSS"]
    page = _create_csspage(page_name, brief, page_args)
    parent = skiboot.from_ident(parent_ident, proj_ident)
    parent.add_page(page, page_ident)


def create_jsonpage(proj_ident, parent_ident, page_ident, page_name, brief, json_data):
    """Builds the json page from the given json string, or ordered dictionary and adds it to project"""
    if isinstance(json_data, str):
        page_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        page_dict = json_data
    if "JSON" not in page_dict:
        raise excepts.ServerError("Invalid file")
    page_args = page_dict["JSON"]
    page = _create_jsonpage(page_name, brief, page_args)
    parent = skiboot.from_ident(parent_ident, proj_ident)
    parent.add_page(page, page_ident)


def create_respondpage(proj_ident, parent_ident, page_ident, page_name, brief, json_data):
    """Builds the respond page from the given json string, or ordered dictionary and adds it to project"""
    if isinstance(json_data, str):
        page_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        page_dict = json_data
    if "RespondPage" not in page_dict:
        raise excepts.ServerError("Invalid file")
    page_args = page_dict["RespondPage"]
    page = _create_respondpage(page_name, brief, page_args, proj_ident)
    parent = skiboot.from_ident(parent_ident, proj_ident)
    parent.add_page(page, page_ident)


def create_folder(proj_ident, parent_ident, addition_number, folder_name, restricted, json_data):
    """Builds the folder and contents from the given json string, or ordered dictionary and adds it to project
       Returns the top folder ident"""
    if isinstance(json_data, str):
        folder_dict = json.loads(json_data, object_pairs_hook=collections.OrderedDict)
    else:
        folder_dict = json_data
    # first check no clash of ident numbers
    project = skiboot.getproject(proj_ident)
    project_numbers = [ident.num for ident in project.identitems.keys()]
    new_numbers = []
    _check_idents_for_create_folder(project_numbers, new_numbers, addition_number, folder_name, folder_dict)
    # all idents ok, now create actual folders and pages and add them to the project

    # get parent
    parent_ident = skiboot.Ident(proj_ident, parent_ident)
    parent = skiboot.get_item(parent_ident)
    if folder_name in parent:
        raise excepts.ServerError("This name already exists in the parent folder")

    # set the parent change value
    parent.change = uuid.uuid4().hex

    if 'brief' in folder_dict:
        brief = folder_dict['brief']
    else:
        brief = "New folder"
    if 'default_page_name' in folder_dict:
        default_page_name = folder_dict['default_page_name']
    else:
        default_page_name = ''
    if not restricted:
        if 'restricted' in folder_dict:
            restricted = folder_dict['restricted']
    topfolder = folder_class_definition.Folder(folder_name, brief, default_page_name, restricted)
    # set the topfolder ident
    topfolder.ident = skiboot.Ident(proj_ident, folder_dict['ident']+addition_number)
    # set the item parentfolder attribute
    topfolder.parentfolder = parent
    item_list = [topfolder]
    # now fill folder contents
    if "folders" in folder_dict:
        for subfolder_name, subfolder in folder_dict['folders'].items():
            item_list.extend(_folder(topfolder, subfolder_name, subfolder, proj_ident, addition_number, restricted))
    if "pages" in folder_dict:
        for page_name, page in folder_dict['pages'].items():
            pageinstance = _page(topfolder, page_name, page, proj_ident, addition_number)
            if pageinstance is not None:
                item_list.append(pageinstance)
    # set this name into the parent
    parent.folders[folder_name] = topfolder.ident
    # item_list has all new folders and pages
    for item in item_list:
        project.identitems[item.ident] = item
    project.clear_cache()
    return topfolder.ident



def _check_idents_for_create_folder(project_numbers, new_numbers, addition_number, folder_name, folder):
    if 'ident' not in folder:
        raise excepts.ServerError("folder ident missing")
    identnum = folder['ident']+addition_number
    if identnum < 1:
        raise excepts.ServerError("Invalid ident number: zero or negative")
    if identnum in project_numbers:
        raise excepts.ServerError("Invalid folder ident number %s for folder %s: already exists in the project" % (identnum, folder_name))
    if identnum in new_numbers:
        raise excepts.ServerError("Invalid folder ident number %s for folder %s: duplicate number" % (identnum, folder_name))
    new_numbers.append(identnum)
    # now do same for sub folders
    if "folders" in folder:
        for subfolder_name, subfolder in folder['folders'].items():
            _check_idents_for_create_folder(project_numbers, new_numbers, addition_number, subfolder_name, subfolder)
    # and check page numbers
    if "pages" in folder:
        for page_name, page in folder['pages'].items():
            if 'ident' not in page:
                raise excepts.ServerError("page ident missing")
            identnum = page['ident']+addition_number
            if identnum < 1:
                raise excepts.ServerError("Invalid ident number: zero or negative")
            if identnum in project_numbers:
                raise excepts.ServerError("Invalid page ident number %s for page %s: already exists in the project" % (identnum, page_name))
            if identnum in new_numbers:
                raise excepts.ServerError("Invalid page ident number %s for page %s: duplicate number" % (identnum, page_name))
            new_numbers.append(identnum)



def read_project(proj_ident, projectfiles):
    "Reads the project.json file and checks version, if ok returns the read project"

    if not proj_ident:
        raise excepts.ServerError("Sorry, the project has not been recognised")

    filepath = os.path.join(projectfiles, proj_ident, "data", "project.json")

    try:
        with open(filepath, 'r') as fp:
            project = json.load(fp, object_pairs_hook=collections.OrderedDict)
    except Exception:
        raise excepts.ServerError("Unable to read file %s" % (filepath,))


    # project created with version = a.b.c
    # this skipole version = d.e.f

    #   The first and second digits may introduce new features
    #   The third digit is only used for refactoring

    # if a > d : The project cannot be read, project created with a later version, which may have introduced new widgets
    # if (a == d) and ( b > e) : The project cannot be read, project created with a later version, which may have introduced new widgets

    # any other combination : the project can be read
    
    if 'skipole' not in project:
        raise excepts.ServerError("Sorry, the project has not been recognised")

    if 'RootFolder' not in project:
        raise excepts.ServerError("Sorry, the project has not been recognised")

    proj_skipole_version = project['skipole']
    proj_tuple_skipole_version = tuple(int(v) for v in proj_skipole_version.split('.'))
    if len(proj_tuple_skipole_version) != 3:
        raise excepts.ServerError("Sorry, the project has not been recognised")
    this_skipole_version = skiboot.version()
    this_tuple_skipole_version = tuple(int(v) for v in this_skipole_version.split('.'))

    if proj_tuple_skipole_version[0] > this_tuple_skipole_version[0]:
        raise excepts.ServerError("Sorry, the project was created with a later version of skipole")

    if (proj_tuple_skipole_version[0] == this_tuple_skipole_version[0]) and (proj_tuple_skipole_version[1] > this_tuple_skipole_version[1]):
        raise excepts.ServerError("Sorry, the project was created with a later version of skipole")

    # update older versions

    # Change multiplier:1 to multiplier:0 in versions 3.1.0 and earlier
    if (proj_tuple_skipole_version[0] <= 3) and (proj_tuple_skipole_version[1] <= 1):
        # get all pages from the rootfolder downwards
        pages = _all_pages(project['RootFolder'])
        for page_dict in pages:
            if "SVG" in page_dict:
                topsvg = page_dict["svg"]
                placeholders = _allparts([[topsvg[0],topsvg[1]]], "SectionPlaceHolder")
                for ph in placeholders:
                    if ("multiplier" in ph) and (ph["multiplier"] == 1):
                        ph["multiplier"] = 0
            elif "TemplatePage" in page_dict:
                tophead = page_dict["body"]
                topbody = page_dict["body"]
                placeholders = _allparts([[tophead[0],tophead[1]]], "SectionPlaceHolder")
                for ph in placeholders:
                    if ("multiplier" in ph) and (ph["multiplier"] == 1):
                        ph["multiplier"] = 0
                placeholders = _allparts([[topbody[0],topbody[1]]], "SectionPlaceHolder")
                for ph in placeholders:
                    if ("multiplier" in ph) and (ph["multiplier"] == 1):
                        ph["multiplier"] = 0
            else:
                continue
    return project



def _all_pages(topfolder):
    """Return list of all page dictionaries in a folder and its sub folders
       Where topfolder is the folder dictionary as read by read_project() """
    # search through folders
    page_list = []
    if "folders" in topfolder:
        for folder in topfolder['folders'].values():
            page_list.extend(_all_pages(folder))
    if "pages" in topfolder:
        for page in topfolder['pages'].values():
            page_list.append(page)
    return page_list


def _allparts(toplist, part_type):
    """Return list of all part dictionaries within toplist that are of type part_type
       Where toplist is the list of [part_type,part_dict] items as read by read_project()"""
    # search through parts
    part_list = []
    for item in toplist:
        item_type, item_dict = item
        if item_type == part_type:
            part_list.append(item_dict)
            if item_type == 'Widget':
                for key in item_dict.keys():
                    if "container_" in key:
                        part_list.extend(_allparts(item_dict[key], part_type))
        elif item_type == 'Part':
            part_list.extend(_allparts(item_dict['parts'], part_type))
        elif item_type == 'Widget':
            for key in item_dict.keys():
                if "container_" in key:
                    part_list.extend(_allparts(item_dict[key], part_type))
    return part_list
 


def create_project(proj_ident, projectfiles):
    """Builds the project from the file project.json, returns a dictionary
          with the following keys
         original_skipole_version
         url
         default_language
         brief
         version
         specialpages - dictionary of label:ident or url
         sections - dictionary of {name:section,..}
         itemlist - list of pages and folders 
         siteroot - the root folder
"""

    project = read_project(proj_ident, projectfiles)
    projectdict = {}

    if 'url' in project:
        projectdict['url'] = project['url']
    else:
        projectdict['url'] = "/"

    if 'default_language' in project:
        projectdict['default_language'] = project['default_language']
    else:
        projectdict['default_language'] = skiboot.default_language()

    if 'brief' in project:
        projectdict['brief'] = project['brief']
    else:
        projectdict['brief'] = "Project %s" % proj_ident

    if 'version' in project:
        projectdict['version'] = project['version']
    else:
        projectdict['version'] = "0.0.0"

    if 'specialpages' in project:
        # dictionary of label:integer or url
        # needs to be a dictionary of label:ident or url
        specialpages = {}
        for label, target in project['specialpages'].items():
            if isinstance(target, int):
                # An ident belonging to this project
                specialpages[label] = skiboot.Ident(proj_ident, target)
            elif ',' in target:
                specialpages[label] = target
            elif '_' in target:
                itemident = skiboot.make_ident(target, proj_ident)
                if itemident is None:
                    specialpages[label] = target
                else:
                    specialpages[label] = itemident
            else:
                # must be a url
                specialpages[label] = target
        projectdict['specialpages'] = specialpages
    else:
        projectdict['specialpages'] = {}

    # ensure all library modules are present
    lib_labels = skiboot.lib_list()
    for lbl in lib_labels:
        if lbl not in projectdict['specialpages']:
            projectdict['specialpages'][lbl] = "lib," + lbl
 

    if 'sections' in project:
        sections = {}
        for section_name, section_part in project['sections'].items():
            section = _create_item(section_part, proj_ident)
            section.widgets = {}
            section.section_places = {}  # currently unused
            embedded = (section_name, '', None)
            section.set_idents(section_name, section.widgets, section.section_places, embedded)
            # now set validator modules in section
            section.load_validator_scriptlinks()
            sections[section_name] = section
        projectdict['sections'] = sections
    else:
        projectdict['sections'] = {}

    # read RootFolder and contents
    rootfolder = project['RootFolder']
    if 'brief' in rootfolder:
        brief = rootfolder['brief']
    else:
        brief = "Root folder"
    if 'default_page_name' in rootfolder:
        default_page_name = rootfolder['default_page_name']
    else:
        default_page_name = 'index'
    siteroot = folder_class_definition.RootFolder(proj_ident, brief, default_page_name)
    # now fill root contents
    item_list = []
    if "folders" in rootfolder:
        for folder_name, folder in rootfolder['folders'].items():
            item_list.extend(_folder(siteroot, folder_name, folder, proj_ident))
    if "pages" in rootfolder:
        for page_name, page in rootfolder['pages'].items():
            pageinstance = _page(siteroot, page_name, page, proj_ident)
            if pageinstance is not None:
                 item_list.append(pageinstance)
    projectdict['itemlist'] = item_list
    # and the root
    projectdict['siteroot'] = siteroot
    return projectdict


def _to_dict(input_dict, proj_ident):
    "If any value in input_dict is an integer, change it to an ident string"
    newdict = {}
    for key, value in input_dict.items():
        if value is True:
            newdict[key] = True
            continue
        if value is False:
            newdict[key] = False
            continue
        if isinstance(value, int):
            newdict[key] = proj_ident + '_' + str(value)
        else:
            newdict[key] = value
    return newdict


def _folder(parent, folder_name, folder_dict, proj_ident, addition_number=0, restricted=False):
    "Create a folder"
    folder_name = folder_name.lower()
    if 'ident' not in folder_dict:
        raise excepts.ServerError("A folder requires an ident")
    ident = skiboot.Ident(proj_ident, folder_dict['ident']+addition_number)
    if 'brief' in folder_dict:
        brief = folder_dict['brief']
    else:
        brief = "New folder"
    if 'default_page_name' in folder_dict:
        default_page_name = folder_dict['default_page_name']
    else:
        default_page_name = 'index'
    if not restricted:
        if 'restricted' in folder_dict:
            restricted = folder_dict['restricted']
    folder = folder_class_definition.Folder(folder_name, brief, default_page_name, restricted)
    # set the folder ident
    folder.ident = ident
    # set the item parentfolder attribute
    folder.parentfolder = parent
    # set this name into the parent
    parent.folders[folder_name] = folder.ident
    page_list = []
    # now fill folder contents
    if "folders" in folder_dict:
        for subfolder_name, subfolder in folder_dict['folders'].items():
            page_list.extend(_folder(folder, subfolder_name, subfolder, proj_ident, addition_number, restricted))
    if "pages" in folder_dict:
        for page_name, page in folder_dict['pages'].items():
            pageinstance = _page(folder, page_name, page, proj_ident, addition_number)
            if pageinstance is not None:
                page_list.append(pageinstance)
    # add folder to page_list
    page_list.append(folder)
    return page_list

def _page(parent, page_name, page_dict, proj_ident, addition_number=0):
    page_name = page_name.lower()
    if 'ident' not in page_dict:
        raise excepts.ServerError("A page ident missing")
    ident = skiboot.Ident(proj_ident, page_dict['ident']+addition_number)
    if 'brief' in page_dict:
        brief = page_dict['brief']
    else:
        brief = ""
    if "FilePage" in page_dict:
        page_args = page_dict["FilePage"]
        page = _create_filepage(page_name, brief, page_args)
    elif "RespondPage" in page_dict:
        page_args = page_dict["RespondPage"]
        page = _create_respondpage(page_name, brief, page_args, proj_ident)
    elif "CSS" in page_dict:
        page_args = page_dict["CSS"]
        page = _create_csspage(page_name, brief, page_args)
    elif "JSON" in page_dict:
        page_args = page_dict["JSON"]
        page = _create_jsonpage(page_name, brief, page_args)
    elif "SVG" in page_dict:
        page_args = page_dict["SVG"]
        page = _create_svgpage(page_name, brief, page_args)
        page.svg = _create_item(page_dict["svg"], proj_ident)
    elif "TemplatePage" in page_dict:
        page_args = page_dict["TemplatePage"]
        page = _create_templatepage(page_name, brief, page_args)
        page.head = _create_item(page_dict["head"], proj_ident)
        page.body = _create_item(page_dict["body"], proj_ident)
    else:
        return
    # set the page ident
    page.ident = ident
    # set the page parentfolder attribute
    page.parentfolder = parent
    # set the name:ident into the parent
    parent.pages[page_name] = page.ident
    if ("TemplatePage" in page_dict) or ("SVG" in page_dict):
        page.set_idents()
    if ("TemplatePage" in page_dict):
        # now set validator modules in page
            page.load_validator_scriptlinks()
    return page


def _create_filepage(page_name, brief, page_args):
    "Create a filepage"
    if 'filepath' in page_args:
        filepath = page_args['filepath']
    else:
        filepath = ''
    if 'mimetype' in page_args:
        mimetype = page_args['mimetype']
    else:
        mimetype = None
    if 'enable_cache' in page_args:
        enable_cache = page_args['enable_cache']
    else:
        enable_cache = False
    return page_class_definition.FilePage(page_name, brief, filepath, mimetype, enable_cache)


def _create_respondpage(page_name, brief, page_args, proj_ident):
    "Create a respondpage"
    responder_name = page_args['class']
    # import module
    mod = importlib.import_module('.ski.responders', 'skipole')
    Responder = None
    for cls in inspect.getmembers(mod, inspect.isclass):
        # cls[0] is responder name, cls[1] is the responder class
        if responder_name == cls[0]:
            Responder = cls[1]
            break
    else:
        raise excepts.ServerError("Responder %s not recognised" % (responder_name,))
    # get the data_args of the responder
    data_args = _to_dict(page_args["original_args"], proj_ident)
    if 'allowed_callers' in data_args:
        # This arg is a list, convert integers to ident strings
        allowed_callers = []
        for value in data_args['allowed_callers']:
            if isinstance(value, int):
                allowed_callers.append(proj_ident + '_' + str(value))
            else:
                allowed_callers.append(value)
        data_args['allowed_callers'] = allowed_callers
    # get the fields of the responder
    fields = dict(page_args["original_fields"])
    # make the responder
    responder=Responder(**data_args)
    # The option of no fields is accepted, as this may be a part constructed page
    if fields:
        responder.set_fields(fields)
    return page_class_definition.RespondPage(page_name, brief, responder)


def _create_csspage(page_name, brief, page_args):
    "Create a CSS page"
    if 'enable_cache' in page_args:
        enable_cache = page_args['enable_cache']
    else:
        enable_cache = False
    # style is an ordered dictionary, no need to convert
    if 'style' in page_args:
        style = page_args["style"]
    else:
        style = {}
    return page_class_definition.CSS(page_name, brief, style, enable_cache)


def _create_jsonpage(page_name, brief, page_args):
    "Create a JSON page"
    if 'enable_cache' in page_args:
        enable_cache = page_args['enable_cache']
    else:
        enable_cache = False
    # content is an ordered dictionary, no need to convert
    if 'content' in page_args:
        content = page_args["content"]
    else:
        content = None
    return page_class_definition.JSON(page_name, brief, content, enable_cache)


def _create_svgpage(page_name, brief, page_args):
    "Create an svg page"
    width = page_args["width"]
    height = page_args["height"]
    if "css_list" in page_args:
        css_list = page_args["css_list"]
    else:
        css_list = []
    if 'enable_cache' in page_args:
        enable_cache = page_args['enable_cache']
    else:
        enable_cache = False
    return page_class_definition.SVG(page_name, brief, width, height, css_list, enable_cache)


def _create_templatepage(page_name, brief, page_args):
    "Create a template page"
    args = {}
    if 'show_backcol' in page_args:
        args['show_backcol'] = page_args["show_backcol"]
    if "last_scroll" in page_args:
        args["last_scroll"] = page_args["last_scroll"]
    if 'interval' in page_args:
        args["interval"] = page_args["interval"]
    if 'interval_target' in page_args:
        args["interval_target"] = page_args["interval_target"]
    if 'lang' in page_args:
        args["lang"] = page_args["lang"]
    if 'backcol' in page_args:
        args["backcol"] = page_args["backcol"]
    if 'catch_to_html' in page_args:
        args["catch_to_html"] = page_args["catch_to_html"]
    if "default_error_widget" in page_args:
        s = page_args["default_error_widget"][0]
        w = page_args["default_error_widget"][1]
        args["default_error_widget"] = skiboot.WidgField(s=s, w=w, f='', i='')
    return page_class_definition.TemplatePage(name=page_name, brief = brief, **args)


def _create_item(item_list, proj_ident):
    "Return the item, be it a Part, ClosedPart etc.."
    itemtype = item_list[0]
    itemdict = item_list[1]
    if itemtype == 'Part':
        return _create_part(itemdict, proj_ident)
    if itemtype == 'ClosedPart':
        return _create_closedpart(itemdict, proj_ident)
    if itemtype == 'Section':
        return _create_section(itemdict, proj_ident)
    if itemtype == 'TextBlock':
        return _create_textblock(itemdict, proj_ident)
    if itemtype == 'SectionPlaceHolder':
        return _create_sectionplaceholder(itemdict, proj_ident)
    if itemtype == 'Widget':
        return _create_widget(itemdict, proj_ident)
    if itemtype == 'ClosedWidget':
        return _create_widget(itemdict, proj_ident)
    if itemtype == 'HTMLSymbol':
        return _create_htmlsymbol(itemdict, proj_ident)
    if itemtype == 'Comment':
        return _create_comment(itemdict, proj_ident)
    if itemtype == 'Text':
        return itemdict


def _create_part(part_dict, proj_ident):
    "Returns a Part"
    tag_name = part_dict["tag_name"]
    if 'brief' in part_dict:
        brief = part_dict["brief"]
    else:
        brief = ''
    show = part_dict["show"]
    hide_if_empty = part_dict["hide_if_empty"]
    if "attribs" in part_dict:
        attribs = dict(part_dict["attribs"])
    else:
        attribs = {}
    # create the part
    part = tag.Part(tag_name, attribs, show=show, hide_if_empty=hide_if_empty, brief=brief)
    # now the Part contents in a list
    parts = part_dict["parts"]
    for index,item_list in enumerate(parts):
        part[index] = _create_item(item_list, proj_ident)
    return part


def _create_closedpart(part_dict, proj_ident):
    "Returns a ClosedPart"
    tag_name = part_dict["tag_name"]
    if 'brief' in part_dict:
        brief = part_dict["brief"]
    else:
        brief = ''
    show = part_dict["show"]
    if "attribs" in part_dict:
        attribs = dict(part_dict["attribs"])
    else:
        attribs = {}
    return tag.ClosedPart(tag_name, attribs, show=show, brief=brief)


def _create_section(part_dict, proj_ident):
    "Returns a Section"
    tag_name = part_dict["tag_name"]
    if 'brief' in part_dict:
        brief = part_dict["brief"]
    else:
        brief = ''
    show = part_dict["show"]
    hide_if_empty = part_dict["hide_if_empty"]
    if "attribs" in part_dict:
        attribs = dict(part_dict["attribs"])
    else:
        attribs = {}
    # create the section
    section = tag.Section(tag_name, attribs, show=show, hide_if_empty=hide_if_empty, brief=brief)
    # now the section contents in a list
    parts = part_dict["parts"]
    for index,item_list in enumerate(parts):
        part = _create_item(item_list, proj_ident)
        # as this is going into a section, remove any placeholders
        _remove_sectionplaceholders(part)
        section[index] = part
    return section



def _create_textblock(part_dict, proj_ident):
    "Creates a TextBlock"
    textref = part_dict["textref"]
    project = ''
    if "project" in part_dict:
        if part_dict["project"] != proj_ident:
            project = part_dict["project"]
    if "failmessage" in part_dict:
        failmessage = part_dict["failmessage"]
    else:
        failmessage=''
    show = part_dict["show"]
    escape = part_dict["escape"]
    linebreaks = part_dict["linebreaks"]
    if "replace_strings" in part_dict:
        replace_strings = part_dict["replace_strings"]
    else:
        replace_strings = []
    return tag.TextBlock(textref=textref, project=project, failmessage=failmessage, escape=escape, linebreaks=linebreaks, replace_strings=replace_strings, show=show)


def _create_sectionplaceholder(part_dict, proj_ident):
    "Creates a sectionplaceholder"
    section_name = part_dict['section_name']
    placename = part_dict['placename']
    if 'brief' in part_dict:
        brief = part_dict['brief']
    else:
        brief = "Section %s with alias %s" % (section_name, placename)
    if 'multiplier' in part_dict:
        multiplier = part_dict['multiplier']
    else:
        multiplier = 0
    if 'mtag' in part_dict:
        mtag = part_dict['mtag']
    else:
        mtag = "div"    
    return tag.SectionPlaceHolder(section_name=section_name, placename=placename, multiplier=multiplier, mtag=mtag, brief=brief)


def _create_htmlsymbol(part_dict, proj_ident):
    "Creates an html symbol"
    text = part_dict['text']
    return tag.HTMLSymbol(text=text)


def _create_comment(part_dict, proj_ident):
    "Creates a comment"
    text = part_dict['text']
    return tag.Comment(text=text)


def _create_widget(part_dict, proj_ident):
    "Returns a Widget"
    mod_name, widg_class_name = part_dict['class'].split(".")
    # import module
    try:
        mod = importlib.import_module('skipole.ski.widgets.'+mod_name)
    except Exception:
        print("ERROR: Unable to import widget %s.%s" % (mod_name, widg_class_name))
        raise excepts.ServerError("Error reading file")
    for cls in inspect.getmembers(mod, inspect.isclass):
        # cls[0] is widget class name, cls[1] is the widget class
        if widg_class_name == cls[0]:
            Widg = cls[1]
            break
    else:
        print("ERROR: Widget %s.%s not found" % (mod_name, widg_class_name))
        raise excepts.ServerError("Error reading file")
    # so Widg contains the widget class, must now instanciate it
    if 'name' in part_dict:
        name = part_dict['name']
    else:
        name = None
    if 'brief' in part_dict:
        brief = part_dict['brief']
    else:
        brief = ''
    if 'fields' in part_dict:
        fields = _to_dict(part_dict['fields'], proj_ident)
    else:
        fields = {}
    widg = Widg(name=name, brief=brief, **fields)
    # widg may have contents
    if widg.can_contain():
        for cont in range(widg.len_containers()):
            container = "container_%s" % (cont,)
            if container in part_dict:
                item_list = part_dict[container]
                # item_list may be an empty list
                if not item_list:
                    # if an empty list, place an empty string in the container
                    widg.append_to_container(cont, '')
                    continue
                # test for list until all projects have containers changed to lists
                if isinstance(item_list[0], list):
                    for itempart in item_list:
                        part = _create_item(itempart, proj_ident)
                        widg.append_to_container(cont, part)
                else:
                    # can be removed once projects converted
                    part = _create_item(item_list, proj_ident)
                    widg.append_to_container(cont, part)
    if "set_names" in part_dict:
        fields_names = part_dict["set_names"]
        for field, name in fields_names.items():
            widg.set_name(field, name)
    if "validators" in part_dict:
        val_dict = part_dict["validators"]
        for field_name, val_list in val_dict.items():
            _get_validators(widg, field_name, val_list, proj_ident)
    return widg


def _get_validators(widget, field_name, val_list, proj_ident):
    "Add validators in val_list to widget.field_name"
    for validator in val_list:
        val_class = validator['class']
        # now write the validator fields
        if "message" in validator:
            message = validator["message"]
        else:
            message=''
        if "message_ref" in validator:
            message_ref = validator["message_ref"]
        else:
            message_ref = ''
        if "displaywidget" in validator:
            displaywidget = validator["displaywidget"]
        else:
            displaywidget = None
        if "allowed_values" in validator:
            allowed_values = validator["allowed_values"]
        else:
            allowed_values = []
        if "val_args" in validator:
            val_args = validator["val_args"]
        else:
            val_args = {}
        mod_name, val_class_name = val_class.split('.')
        try:
            mod = importlib.import_module('skipole.ski.validators.'+mod_name)
        except Exception:
            print("ERROR: Unable to import validator %s.%s" % (mod_name, val_class_name))
            raise excepts.ServerError("Error reading file")
        # get Validator class
        for cls in inspect.getmembers(mod, inspect.isclass):
            # cls[0] is validator class name, cls[1] is the validator class
            if val_class_name == cls[0]:
                Val = cls[1]
                break
        else:
            print("ERROR: Validator %s.%s not found" % (mod_name, val_class_name))
            raise excepts.ServerError("Error reading file")
        # create the validator
        val = Val(message=message, message_ref=message_ref, displaywidget=displaywidget, allowed_values=allowed_values, **val_args)
        # add validator to widget
        widget.add_validator(field_name, val)






from skipole import FailPage, ValidateError, GoTo, ServerError, skilift

from skipole.skilift import editsection


def insert_in_section(skicall):
    """Called by domtable to either insert or append an item in a section
       sets page_data to populate the insert or append modal panel"""
    call_data = skicall.call_data
    page_data = skicall.page_data
    if ('editdom', 'domtable', 'contents') not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data['editdom', 'domtable', 'contents']
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        if len(location_list) == 1:
            # no location integers, so location_list[0] is the section name
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[1:] )
        section_name = location_list[0]
    except:
        raise FailPage("Item to append to has not been recognised")

    # location is a tuple of section_name, None for no container, tuple of location integers
    location = (section_name, None, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, None, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    if location_integers:
        insert_location = section_name + '-' + '-'.join(str(i) for i in location_integers)
    else:
        insert_location = section_name

    # display the modal panel
    page_data[("sectioninserts","insertitem","hide")] = False

    if (part_tuple.part_type == "Part") or (part_tuple.part_type == "Section"):
        # insert
        page_data[("sectioninserts","insertpara","para_text")] = "Choose an item to insert"
        page_data[("sectioninserts","insertupload","para_text")] = "Or insert a new block by uploading a block definition file:"
    else:
        # append
        page_data[("sectioninserts","insertpara","para_text")] = "Choose an item to append"
        page_data[("sectioninserts","insertupload","para_text")] = "Or append a new block by uploading a block definition file:"

    # for each of the links, set get_field1 to be the insert_location
    page_data[("sectioninserts","insert_text","get_field1")] = insert_location
    page_data[("sectioninserts","insert_textblock","get_field1")] = insert_location
    page_data[("sectioninserts","insert_symbol","get_field1")] = insert_location
    page_data[("sectioninserts","insert_comment","get_field1")] = insert_location
    page_data[("sectioninserts","insert_element","get_field1")] = insert_location
    page_data[("sectioninserts","insert_widget","get_field1")] = insert_location

    # set the hidden field
    page_data[("sectioninserts","uploadpart","hidden_field1")] = insert_location



def insert_text(skicall):
    "Inserts text into a section"
    call_data = skicall.call_data
    page_data = skicall.page_data

    if ("sectioninserts","insert_text","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data["sectioninserts","insert_text","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        if len(location_list) == 1:
            # no location integers, so location_list[0] is the section name
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[1:] )
        section_name = location_list[0]
    except:
        raise FailPage("Item to append to has not been recognised")

    # location is a tuple of section_name, None for no container, tuple of location integers
    location = (section_name, None, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, None, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    call_data['section_name'] = section_name

    new_text = 'Set text here'

    call_data['schange'], new_location = skilift.insert_item_in_section(editedprojname, section_name, call_data['schange'], location, new_text)
    call_data['location'] = new_location
    
    # go to edit text page
    page_data[("adminhead","page_head","large_text")] = "Edit Text in section : %s" % (section_name,)
    # Set the text in the text area
    page_data[("text_input","input_text")] = new_text


def insert_textblock(skicall):
    "Fills the template page for creating a textblock reference which will be inserted in the section"
    call_data = skicall.call_data
    page_data = skicall.page_data

    if ("sectioninserts","insert_textblock","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data["sectioninserts","insert_textblock","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        if len(location_list) == 1:
            # no location integers, so location_list[0] is the section name
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[1:] )
        section_name = location_list[0]
    except:
        raise FailPage("Item to append to has not been recognised")

    # location is a tuple of section_name, None for no container, tuple of location integers
    location = (section_name, None, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, None, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    # section_name and location are set into call_data, so they will also be set into the ident data
    call_data['section_name'] = section_name
    call_data['location'] = location

    # and set page data for the template page which inserts an textblock reference

    page_data[("adminhead","page_head","large_text")] = "Insert TextBlock in section %s" % (section_name,)

    page_data[("linebreaks","radio_values")]=['ON', 'OFF']
    page_data[("linebreaks","radio_text")]=['On', 'Off']
    page_data[("linebreaks","radio_checked")] = 'ON'

    page_data[("setescape","radio_values")]=['ON', 'OFF']
    page_data[("setescape","radio_text")]=['On', 'Off']
    page_data[("setescape","radio_checked")] = 'ON'



def insert_symbol(skicall):
    "Inserts html symbol into a section"
    call_data = skicall.call_data
    page_data = skicall.page_data

    if ("sectioninserts","insert_symbol","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data["sectioninserts","insert_symbol","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        if len(location_list) == 1:
            # no location integers, so location_list[0] is the section name
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[1:] )
        section_name = location_list[0]
    except:
        raise FailPage("Item to append to has not been recognised")

    # location is a tuple of section_name, None for no container, tuple of location integers
    location = (section_name, None, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, None, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    call_data['section_name'] = section_name
    call_data['schange'], new_location = editsection.create_html_symbol_in_section(editedprojname, section_name, call_data['schange'], location)
    call_data['location'] = new_location
    
    # go to edit symbol page
    sym = editsection.get_symbol(editedprojname, section_name, call_data['schange'], new_location)
    page_data[("adminhead","page_head","large_text")] = "Edit Symbol in section : %s" % (section_name,)
    page_data["symbol_input","input_text"] = sym


def insert_comment(skicall):
    "Inserts a comment into a section"
    call_data = skicall.call_data
    page_data = skicall.page_data

    if ("sectioninserts","insert_comment","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data["sectioninserts","insert_comment","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        if len(location_list) == 1:
            # no location integers, so location_list[0] is the section name
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[1:] )
        section_name = location_list[0]
    except:
        raise FailPage("Item to append to has not been recognised")

    # location is a tuple of section_name, None for no container, tuple of location integers
    location = (section_name, None, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, None, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    call_data['section_name'] = section_name
    call_data['schange'], new_location = editsection.create_html_comment_in_section(editedprojname, section_name, call_data['schange'], location)
    call_data['location'] = new_location
    
    # go to edit comment page
    com = editsection.get_comment(editedprojname, section_name, call_data['schange'], new_location)
    page_data[("adminhead","page_head","large_text")] = "Edit Comment in section : %s" % (section_name,)
    page_data[("comment_input","input_text")] = com


def insert_element(skicall):
    "Fills the template page for creating an html element which will be inserted in the section"
    call_data = skicall.call_data
    page_data = skicall.page_data

    if ("sectioninserts","insert_element","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data["sectioninserts","insert_element","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        if len(location_list) == 1:
            # no location integers, so location_list[0] is the section name
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[1:] )
        section_name = location_list[0]
    except:
        raise FailPage("Item to append to has not been recognised")

    # location is a tuple of section_name, None for no container, tuple of location integers
    location = (section_name, None, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, None, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    # section_name and location are set into call_data, so they will also be set into the ident data
    call_data['section_name'] = section_name
    call_data['location'] = location

    # and set page data for the template page which inserts an element
    page_data[("adminhead","page_head","large_text")] = "Insert an HTML element into section " + call_data['section_name']


def insert_widget(skicall):
    "Gets section_name and location, used for creating a widget which will be inserted in the section"
    call_data = skicall.call_data
    page_data = skicall.page_data

    if ("sectioninserts","insert_widget","get_field1") not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data["sectioninserts","insert_widget","get_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        if len(location_list) == 1:
            # no location integers, so location_list[0] is the section name
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[1:] )
        section_name = location_list[0]
    except:
        raise FailPage("Item to append to has not been recognised")

    # location is a tuple of section_name, None for no container, tuple of location integers
    location = (section_name, None, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, None, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    # section_name and location are set into call_data, so they will also be set into the ident data
    call_data['section_name'] = section_name
    call_data['location'] = location

    # at this point, the call is passed to 54507 which is a responder which lists widget modules
    # and displays them on a template


def insert_upload(skicall):
    "Gets section_name and location, used for creating a widget which will be inserted in the section"
    call_data = skicall.call_data
    page_data = skicall.page_data

    if ("sectioninserts","uploadpart","hidden_field1") not in call_data:
        raise FailPage(message = "item to edit missing")
    editedprojname = call_data['editedprojname']
    part = call_data["sectioninserts","uploadpart","hidden_field1"]
    location_list = part.split('-')
    # first item should be a string, rest integers
    try:
        if len(location_list) == 1:
            # no location integers, so location_list[0] is the section name
            location_integers = ()
        else:
            location_integers = tuple( int(i) for i in location_list[1:] )
        section_name = location_list[0]
    except:
        raise FailPage("Item to append to has not been recognised")

    # location is a tuple of section_name, None for no container, tuple of location integers
    location = (section_name, None, location_integers)
    # get part_tuple from project, pagenumber, section_name, location
    part_tuple = skilift.part_info(editedprojname, None, section_name, location)
    if part_tuple is None:
        raise FailPage("Item to append to has not been recognised")

    # section_name and location are set into call_data, so they will also be set into the ident data
    call_data['section_name'] = section_name

    # get file contents
    file_contents = call_data["sectioninserts","uploadpart", "action"]
    json_string = file_contents.decode(encoding='utf-8')
    try:
        call_data['schange'] = editsection.create_part_in_section(editedprojname, section_name, call_data['schange'], location, json_string)
    except ServerError as e:
        if e.message:
            raise FailPage(e.message)
        else:
            raise FailPage("An error has occurred in creating the item")

    call_data['status'] = 'New block created'


